// Copyright 2015 CoreOS, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package timeutil

import "time"

// UnixNanoToTime returns the local time corresponding to the given Unix time in nanoseconds.
// If the given Unix time is zero, an uninitialized zero time is returned.
func UnixNanoToTime(ns int64) time.Time {
	var t time.Time
	if ns != 0 {
		t = time.Unix(0, ns)
	}
	return t
}
